"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable-next-line:no-var-requires
var plantuml = require("node-plantuml"); // there are no type definitions for this module :-(
var fs = require("fs");
var path = require("path");
/**
 * Generates local image files.
 */
var ImageGenerator = /** @class */ (function () {
    function ImageGenerator() {
        /** Stamp used for every image file. */
        this.fileNameStamp = "umlClassDiagram";
        /** Number of generated image files. */
        this.numberOfGeneratedImages = 0;
    }
    /**
     * Sets the output directory.
     * @param outputDirectory The path to the folder into which the files are generated.
     */
    ImageGenerator.prototype.setOutputDirectory = function (outputDirectory) {
        this.outputDirectory = outputDirectory;
    };
    /**
     * Writes a class diagram as a local image to the disc.
     * @param plantUml The PlantUML code for the image.
     * @param imageName The name of the class diagram.
     * @param imageFormat The format of the image to generate. (eg.: "png" or "svg")
     * @returns Absolute path to the written file.
     */
    ImageGenerator.prototype.writeImageToFile = function (plantUml, imageName, imageFormat) {
        var filename = this.createFilenameForNextImage(imageName, imageFormat);
        var absoluteFilename = path.join(this.outputDirectory, filename);
        var gen = plantuml.generate(plantUml, { format: imageFormat });
        gen.out.pipe(fs.createWriteStream(absoluteFilename));
        ++this.numberOfGeneratedImages;
        return absoluteFilename;
    };
    /**
     * Promises to write a class diagram as an image into a buffer.
     * @param plantUml The PlantUML code for the image.
     * @param imageFormat The format of the image to generate. (eg.: "png" or "svg")
     * @returns The promise of a buffer including the image of the class diagram.
     */
    ImageGenerator.prototype.writeImageToBuffer = function (plantUml, imageFormat) {
        return new Promise(function (resolve, reject) {
            var chunks = new Array();
            var gen = plantuml.generate(plantUml, { format: imageFormat });
            gen.out.on("data", function (chunk) { return chunks.push(chunk); });
            gen.out.on("error", function (e) { return reject(e); });
            gen.out.on("end", function () { return resolve(Buffer.concat(chunks)); });
        });
    };
    /**
     * Returns the PlantUML website URL that generates a UML diagram.
     * @param encodedPlantUml The encoded PlantUML for the diagram.
     * @param imageFormat The format of the image the URL should generate. (eg.: "png" or "svg")
     * @returns The URL that generates the diagram.
     */
    ImageGenerator.prototype.createPlantUmlServerUrl = function (encodedPlantUml, imageFormat) {
        return "http://www.plantuml.com/plantuml/" + imageFormat + "/" + encodedPlantUml;
    };
    /**
     * Creates a filename for the next image.
     * @param imageName A name of the thing visible on the image.
     * @param fileExtension Extension used in the filename.
     * @returns The generated filename.
     */
    ImageGenerator.prototype.createFilenameForNextImage = function (imageName, fileExtension) {
        return imageName + "-" + this.fileNameStamp + "-" + this.numberOfGeneratedImages + "." + fileExtension;
    };
    return ImageGenerator;
}());
exports.ImageGenerator = ImageGenerator;
